<?php

namespace Drupal\cacheexclude\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Contribute form.
 */
class AdminSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cacheexclude_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'cacheexclude.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('cacheexclude.settings');
    $selected_types = $config->get('cacheexclude_node_types') ?? [];

    $form['cacheexclude_list'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Pages to exclude from caching'),
      '#default_value' => $config->get('cacheexclude_list'),
      '#width' => 40,
      '#height' => 10,
      '#description' => $this->t("Enter one page per line as Drupal paths. Begin link with leading slash. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.",
        ['%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>']
      ),
    ];

    $form['cacheexclude_node_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Content types to exclude from caching'),
      '#default_value' => array_combine($selected_types, $selected_types),
      '#options' => node_type_get_names(),
      '#description' => $this->t("Check all content types that you do not want to be cached."),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Clear the page cache when new settings are added.
    drupal_flush_all_caches();

    $config = $this->configFactory->getEditable('cacheexclude.settings');
    $values = $form_state->getValue('cacheexclude_node_types');
    $enabled = array_keys(array_filter($values));
    $config->set('cacheexclude_list', $form_state->getValue('cacheexclude_list'))->save();
    $config->set('cacheexclude_node_types', $enabled)->save();
    parent::submitForm($form, $form_state);
  }

}
