<?php

namespace Drupal\Tests\cacheexclude\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\system\Functional\Cache\AssertPageCacheContextsAndTagsTrait;

/**
 * Test CacheExclude on 404 pages.
 *
 * @group cacheexclude
 */
class CacheExclude404Test extends BrowserTestBase {

  use AssertPageCacheContextsAndTagsTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['cacheexclude', 'page_cache', 'node'];

  /**
   * Node ID for 404 page.
   *
   * @var int
   */
  protected int $id;

  /**
   * Setup cacheexclude setting.
   */
  protected function setUp(): void {
    parent::setUp();
    $this->drupalCreateContentType(['type' => 'page']);
    $this->id = $this->drupalCreateNode()->id();

    // Set 404 page.
    $this->config('system.site')->set('page.404', '/node/' . $this->id)->save();

    // Enable page cache.
    $this->config('system.performance')->set('cache.page.max_age', 300)->save();

    drupal_flush_all_caches();
  }

  /**
   * Tests that cacheexclude works as expected.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testCacheExclude404(): void {
    $path = 'trigger_404';

    $this->drupalGet($path);
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'MISS');
    // Second request should HIT.
    $this->drupalGet($path);
    $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'HIT');

    // Set Cache Exclude to 404 page.
    $this->config('cacheexclude.settings')->set('cacheexclude_list', '/node/' . $this->id)->save();
    drupal_flush_all_caches();

    $this->drupalGet($path);
    if (str_starts_with(\Drupal::VERSION, '11.0')) {
      // See https://www.drupal.org/node/2958442
      $this->assertSession()->responseHeaderDoesNotExist('X-Drupal-Cache');
    }
    else {
      // See https://www.drupal.org/node/2958442
      $this->assertSession()->responseHeaderEquals('X-Drupal-Cache', 'UNCACHEABLE (response policy)');
    }
  }

}
